@@ -19,8 +19,47 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-/// A trait for defining backtracking behaviour over
-/// This serves to generify the copying and non-copying verions and their behaviour
+/// A historical record representation
+pub trait Record {
+  /// The type used to refer to positions in the history
+  type RefPoint;
+  
+  /// Yield a reference to the current point in the history
+  /// This reference must be valid for as long as the current
+  /// point remains in the history
+  fn get_ref_point(&self) -> Self::RefPoint;
+
+  /// Eliminate all the values before the given reference point from the history
+  fn forget_before(&mut self, point: Self::RefPoint);
+
+  /// Forget all the values before the current position in the iterator
+  /// ```
+  /// extern crate backtracking_iterator;
+  /// use backtracking_iterator::{Record, BacktrackingIterator};
+  ///
+  /// let v = vec![1_u8, 2_u8];
+  /// let mut rec = backtracking_iterator::BacktrackingRecorder::new(v.into_iter());
+  /// {
+  ///   let mut bt = rec.copying();
+  ///   bt.next();
+  /// }
+  ///
+  /// //Before we call this, 1_u8 is in the history
+  /// rec.forget();
+  ///
+  /// {
+  ///   let mut bt = rec.copying();
+  ///   assert!(bt.next().unwrap() == 2_u8);
+  /// }
+  /// ```
+  fn forget(&mut self) {
+    let now = self.get_ref_point();
+    self.forget_before(now);
+  }
+}
+
+/// An iterator capable of backtracking behaviour
+/// This generifies the copying and non-copying versions and their behaviour
 pub trait BacktrackingIterator: Iterator {
   /// The type used to refer to positions in the history
   type RefPoint;
@@ -52,6 +91,7 @@ pub trait BacktrackingIterator: Iterator {
   /// 
   /// bt.backtrack(wb_pos);
   /// assert!(bt.next().unwrap() == 2_u8);
+  /// ```
   fn backtrack(&mut self, point: Self::RefPoint);
 
   /// Start the iterator again from all the elements in the current history
@@ -73,37 +113,15 @@ pub trait BacktrackingIterator: Iterator {
     let oldest = self.get_oldest_point();
     self.backtrack(oldest);
   }
-
-  /// Eliminate all the values before the given reference point from the history
-  fn forget_before(&mut self, point: Self::RefPoint);
-
-  /// Forget all the values before the current position in the iterator
-  /// ```
-  /// extern crate backtracking_iterator;
-  /// use backtracking_iterator::BacktrackingIterator;
-  ///
-  /// let v = vec![1_u8, 2_u8];
-  /// let mut rec = backtracking_iterator::BacktrackingRecorder::new(v.into_iter());
-  /// let mut bt = rec.copying();
-  /// bt.next();
-  ///
-  /// //Before we call this, 1_u8 is in the history
-  /// bt.forget();
-  ///
-  /// bt.start_again();
-  /// assert!(bt.next().unwrap() == 2_u8);
-  /// ```
-  fn forget(&mut self) {
-    let now = self.get_ref_point();
-    self.forget_before(now);
-  }
 }
 
-/// A trait for an iterator that can be walked back on, parameterised for a lifetime
+/// An iterator that can be walked back on, parameterised for a lifetime
 /// This trait is a workaround for the lack of generic associated types - it is expected
 /// to be implemented `for` every lifetime, for reasons of utility
 pub trait Walkbackable<'history> {
+  /// The type used to refer to positions in the history
   type RefPoint;
+  /// The type of item in the history
   type Item;
 
   /// The type of the walk-back iterator 
@@ -127,14 +145,15 @@ pub trait Walkbackable<'history> {
   fn walk_back(&'history self) -> Self::Walkback;
 }
 
-/// A trait for walking back over a backtracking history
+/// An iterator which walks back over a history
 pub trait Walkback<'history>: Iterator {
   /// The type used to refer to positions in the history
   type RefPoint;
 
-  /// Yield a reference to the current point in the history
+  /// Yield a reference to the current point in the history -
+  /// the current position is before the most-recently-yielded element.
   /// This reference must be valid in the parent BacktrackingIterator,
-  /// and must remain valid until the next call to `next()` on this iterator
+  /// and must remain valid for as long as the walkback exists
   fn get_ref_point(&self) -> Self::RefPoint;
 }
 
